/*
 *
 * @Component			OMAPCONF
 * @Filename			pmi44xx_voltdm.c
 * @Description			OMAP4 PMI Voltage Domain OPP Change Trace
 *				Decoding library
 * @Author			Patrick Titiano (p-titiano@ti.com)
 * @Date			2011
 * @Copyright			Texas Instruments Incorporated
 *
 *
 * Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#include <pmi44xx_voltdm.h>
#include <stdio.h>
#include <autoadjust_table.h>
#include <lib.h>
#include <stm44xx.h>
#include <pmic.h>

/* #define PMI44XX_VOLTDM_DEBUG */
#ifdef PMI44XX_VOLTDM_DEBUG
	#define dprintf(format, ...)	 printf(format, ## __VA_ARGS__)
#else
	#define dprintf(format, ...)
#endif


#define PMI44XX_VOLTDM_NAME_MAX 11


typedef struct {
	/* Voltage in integer format, as sent to PMIC SMPS */
	unsigned int vsel[PMI44XX_VOLTDM_LOGIC_MAX];
	/* Relative timestamp, in micro-seconds */
	double ts;
} pmi44xx_voltdm_transition;


#ifdef PMI44XX_VOLTDM_DEBUG
static const char
	pmi44xx_logic_voltdm_names_table
		[PMI44XX_VOLTDM_LOGIC_MAX][PMI44XX_VOLTDM_NAME_MAX] = {
		"LOGIC_MPU",
		"LOGIC_IVA",
		"LOGIC_CORE"};
#endif


#if 0 /* defined for future use */
static const char
	pmi44xx_mem_voltdm_names_table
		[PMI44XX_VOLTDM_MEM_MAX][PMI44XX_VOLTDM_NAME_MAX] = {
		"MEM_MPU",
		"MEM_IVA",
		"MEM_CORE"};
#endif


/* ------------------------------------------------------------------------*//**
 * @FUNCTION		pmi44xx_voltdm_transitions_extract
 * @BRIEF		extract voltage domain voltage transition(s) from PMI
 *			trace file.
 * @RETURNS		0 no voltage domain voltage transition(s) found in PMI
 *			trace file
 *			>0 total number of events extracted from PMI trace file
 *			OMAPCONF_ERR_ARG
 *			OMAPCONF_ERR_NOT_AVAILABLE
 *			OMAPCONF_ERR_UNEXPECTED
 * @param[in,out]	filename: PMI trace file generated by
 *				pmi44xx_trace_decode()
 * @param[in,out]	transitions: list of voltage domain voltage
 *				transition(s) extracted
 *		        from PMI trace file.
 * @DESCRIPTION		extract voltage domain voltage transition(s) from PMI
 *			trace file.
 *//*------------------------------------------------------------------------ */
int pmi44xx_voltdm_transitions_extract(char *filename, genlist *transitions)
{
	unsigned int evt_class, evt_state_lsb, evt_state_msb;
	unsigned int dom, shift, vsels, vsel;
	FILE *fp;
	char line[256];
	char dummy[7], dummy2[8];
	pmi44xx_voltdm_transition voltdm_tr;
	unsigned int evt_count;
	#ifdef PMI44XX_VOLTDM_DEBUG
	unsigned long uv;
	char *name;
	#endif
	double ts = 0.0;
	double volt;
	unsigned int prev_vsel[PMI44XX_VOLTDM_LOGIC_MAX];


	if (transitions == NULL) {
		fprintf(stderr, "%s(): transitions == NULL!\n", __func__);
		return OMAPCONF_ERR_ARG;
	}
	if (filename == NULL) {
		fprintf(stderr, "%s(): filename == NULL!\n", __func__);
		return OMAPCONF_ERR_ARG;
	}

	fp = fopen(filename, "r");
	if (fp == NULL) {
		fprintf(stderr, "%s(): could not open %s!\n",
			__func__, filename);
		return OMAPCONF_ERR_NOT_AVAILABLE;
	}

	genlist_init(transitions);
	evt_count = 0;

	/*
	 * OMAP4430 WORKAROUND: if the voltage domain does not change while
	 * PMI is running, PMI does not have any value to report for that
	 * particular domain. It seems to report 0x8C in such cases.
	 * 0x8C is also sometimes returned in other domains, between 2 samples.
	 * It may be due to some other know bugs like when 2 identical events
	 * happen simultaneously?
	 * SOLUTION:
	 *  1- get initial voltage from VP using voltdm44xx_get_voltage().
	 *  2- save previous vsel reported by PMI
	 *  3- Replace any 0x8C found by either the initial voltage or the last
	 *     good value reported by PMI.
	 */
	for (dom = 0; dom < PMI44XX_VOLTDM_LOGIC_MAX; dom++) {
		voltdm44xx_get_voltage((voltdm44xx_id) (dom + 1), &volt);
		prev_vsel[dom] = smps_uvolt2vsel(vdd_id2smps_id(dom + 1),
			(unsigned int) (volt * 1000000));
	}

	dprintf("pmi_voltdm_events_get(): "
		"parsing trace file...\n");
	while (fgets(line, sizeof(line), fp) != NULL) {
		/* Looking for D8 headers */
		if (strstr(line, "D8 (0x4)") == NULL)
			continue;

		dprintf("new D8 found\n");
		/* Retrieve Event Class */
		if (fgets(line, sizeof(line), fp) == NULL) {
			printf("unexpected end of line reached!\n");
			fclose(fp);
			return OMAPCONF_ERR_UNEXPECTED;
		}
		if (sscanf(line, "\tEvent Class is %s %s Domain (0x%02X)",
			dummy, dummy2, &evt_class) != 3) {
			printf("could not get class from %s", line);
			fclose(fp);
			return OMAPCONF_ERR_UNEXPECTED;
		}
		if (evt_class >= PMI_EVENT_CLASS_MAX) {
			printf("unexpected pm class found! (%d)\n", evt_class);
			fclose(fp);
			return OMAPCONF_ERR_UNEXPECTED;
		}
		dprintf("event class is %d\n", evt_class);

		/* Retrieve event data */
		if (fgets(line, sizeof(line), fp) == NULL) {
			printf("unexpected end of line reached!\n");
			fclose(fp);
			return OMAPCONF_ERR_UNEXPECTED;
		}
		if ((evt_class == PMI_PWRDM_CLASS_LOGIC) ||
			(evt_class == PMI_VOLTDM_CLASS_MEM)) {
			if (sscanf(line, "\t\tEvent Data = 0x%08X",
				&evt_state_lsb) != 1) {
				printf("could not get state from %s", line);
				fclose(fp);
				return OMAPCONF_ERR_UNEXPECTED;
			}
			dprintf("event data is 0x%08X\n", evt_state_lsb);
			evt_state_msb = 0;
		} else {
			if (sscanf(line, "\t\tEvent Data = 0x%08X%08X",
				&evt_state_msb, &evt_state_lsb) != 2) {
				printf("could not get state from %s", line);
				fclose(fp);
				return OMAPCONF_ERR_UNEXPECTED;
			}
			dprintf("event data is 0x%08X%08X\n",
				evt_state_msb, evt_state_lsb);
		}

		if (evt_class == PMI_VOLTDM_CLASS_LOGIC) {
			/* Retrieve all logic domain voltages from event data */
			for (dom = 0; dom < PMI44XX_VOLTDM_LOGIC_MAX; dom++) {
				#ifdef PMI44XX_VOLTDM_DEBUG
				name = (char *)
					pmi44xx_logic_voltdm_names_table[dom];
				#endif
				switch (dom) {
				case PMI44XX_VOLTDM_LOGIC_MPU:
					vsels = evt_state_lsb;
					shift = 0;
					vsel = (vsels & (0xFF << shift))
						>> shift;
					if (vsel == 0x8C)
						vsel = prev_vsel[dom];
					#ifdef PMI44XX_VOLTDM_DEBUG
					uv = smps_vsel2uvolt(
						vdd_id2smps_id(OMAP4_VDD_MPU),
						vsel);
					#endif
					break;
				case PMI44XX_VOLTDM_LOGIC_IVA:
					vsels = evt_state_lsb;
					shift = 16;
					vsel = (vsels & (0xFF << shift))
						>> shift;
					if (vsel == 0x8C)
						vsel = prev_vsel[dom];
					#ifdef PMI44XX_VOLTDM_DEBUG
					uv = smps_vsel2uvolt(
						vdd_id2smps_id(OMAP4_VDD_IVA),
						vsel);
					#endif
					break;
				case PMI44XX_VOLTDM_LOGIC_CORE:
					vsels = evt_state_msb;
					shift = 0;
					vsel = (vsels & (0xFF << shift))
						>> shift;
					if (vsel == 0x8C)
						vsel = prev_vsel[dom];
					#ifdef PMI44XX_VOLTDM_DEBUG
					uv = smps_vsel2uvolt(
						vdd_id2smps_id(OMAP4_VDD_CORE),
						vsel);
					#endif
					break;
				default:
					fprintf(stderr, "%s(): unexpected "
						"domain id (%u)!\n",
						__func__, dom);
					return OMAPCONF_ERR_UNEXPECTED;
				}
				dprintf("dom=%s, shift=%u, vsels=0x%08X, "
					"vsel=0x%X (%1.3lfV)\n",
					name, shift, vsels, vsel,
					(double)
					((double) uv / (double) 1000000.0));
				voltdm_tr.vsel[dom] = vsel;
				prev_vsel[dom] = vsel;
			}
			/* Add new element to list */
			genlist_addtail(transitions, &voltdm_tr,
				sizeof(pmi44xx_voltdm_transition));
			evt_count++;

			dprintf("new event added to the list.\n");
		} else {
			dprintf("event is not voltage domain event, "
				"do not add it to the list.\n");

		}

		/* Retrieve relative timestamp from next line */
		if (fgets(line, sizeof(line), fp) == NULL) {
			printf("unexpected end of line reached!\n");
			fclose(fp);
			return OMAPCONF_ERR_UNEXPECTED;
		}
		if (sscanf(line, "\tTimestamp (relative) = %lfus", &ts) != 1) {
			printf("could not get timestamp from %s", line);
			fclose(fp);
			return OMAPCONF_ERR_UNEXPECTED;
		}
		voltdm_tr.ts += ts;
		dprintf("timestamp (abs) = %lfus\n", voltdm_tr.ts);
	}
	fclose(fp);
	dprintf("pmi_voltdm_events_get(): found %d voltdm events.\n",
		evt_count);

	return evt_count;
}


/* ------------------------------------------------------------------------*//**
 * @FUNCTION		pmi44xx_voltdm_transitions_save
 * @BRIEF		Save voltage domain voltage transitions into files.
 * @RETURNS		0 in case of success
 *			OMAPCONF_ERR_ARG
 *			OMAPCONF_ERR_NOT_AVAILABLE
 *			OMAPCONF_ERR_UNEXPECTED
 * @param[in,out]	transitions: list of voltage transitions for each domain
 * @param[in]		duration: total trace duration in microseconds.
 * @DESCRIPTION		1- Save voltage domain voltage transitions into files.
 *			2- Generate GNUPlot scripts to draw and save charts.
 *//*------------------------------------------------------------------------ */
int pmi44xx_voltdm_transitions_save(genlist *transitions, double duration)
{
	unsigned int dom;
	voltdm44xx_id vdd_id;
	pmi44xx_voltdm_transition tr;
	int count, evt;
	FILE *fp = NULL;
	unsigned long volt;
	char pltfile[64];
	char s[16];
	static char datafile[64] = "omap_vdd_trace.dat";
	static const char draw_plt_file[64] =
		"gnuplot_draw_voltage_charts.plt";
	static const char jpeg_plt_file[64] =
		"gnuplot_create_voltage_traces_jpg.plt";
	static const char jpeg_file[] = "voltage_traces.jpg";

	if (transitions == NULL) {
		printf("%s(): "
			"transitions == NULL!\n", __func__);
		return OMAPCONF_ERR_ARG;
	}

	fp = workdir_fopen(datafile, "w");
	if (fp == NULL) {
		fprintf(stderr, "%s(): could not create %s!\n",
			__func__, datafile);
		return OMAPCONF_ERR_NOT_AVAILABLE;
	}

	/* Save trace into file */
	fprintf(fp, "# OMAP4430 Voltage Domain Transitions Log\n\n");
	fprintf(fp, "# ----------- Trace DURATION = %.3lfms -----------\n\n",
		duration / 1000.0);
	fprintf(fp, "# ----------- Trace FORMAT -----------\n");
	fprintf(fp, "# Timestamp\tVDD_MPU\tVDD_IVA\tVDD_CORE\n");
	fprintf(fp, "# Timestamp is in milliseconds, voltages in volts\n\n");
	fprintf(fp, "# ----------- Trace START -----------\n");

	/* First element in list is the initial state */
	count = genlist_getcount(transitions);
	dprintf("%s(): %u voltage transitions to save.\n", __func__, count);
	for (evt = 0; evt < count; evt++) {
		genlist_get(transitions, evt, &tr);
		sprintf(s, "%.3lf", tr.ts / 1000.0);
		fprintf(fp, "%-8s", s);
		for (dom = 0; dom < PMI44XX_VOLTDM_LOGIC_MAX; dom++) {
			switch (dom) {
			case PMI44XX_VOLTDM_LOGIC_MPU:
				vdd_id = OMAP4_VDD_MPU;
				break;
			case PMI44XX_VOLTDM_LOGIC_IVA:
				vdd_id = OMAP4_VDD_IVA;
				break;
			case PMI44XX_VOLTDM_LOGIC_CORE:
				vdd_id = OMAP4_VDD_CORE;
				break;
			default:
				/* cannot happen */
				printf("%s(): dom (%u) >= "
					"PMI44XX_VOLTDM_LOGIC_MAX\n",
					__func__, dom);
				fclose(fp);
				return OMAPCONF_ERR_UNEXPECTED;
			}
			volt = smps_vsel2uvolt(
				vdd_id2smps_id(vdd_id),
				tr.vsel[dom]);
			sprintf(s, "%.3lf",
				(double) ((double) volt / (double) 1000000.0));
			fprintf(fp, "\t%-5s", s);
		}
		fprintf(fp, "\n");
	}
	fprintf(fp, "# ------------ Trace END ------------\n");
	fclose(fp);
	printf("Voltage transitions saved into \"%s\" file.\n", datafile);

	/* Generate GNUPlot script files to draw and save into jpeg charts */
	for (count = 0; count < 2; count++) {
		if (count == 0)
			strcpy(pltfile, draw_plt_file);
		else
			strcpy(pltfile, jpeg_plt_file);
		fp = workdir_fopen(pltfile, "w");
		if (fp == NULL) {
			fprintf(stderr, "%s(): could not create %s!\n",
				__func__, pltfile);
			return OMAPCONF_ERR_NOT_AVAILABLE;
		}

		fprintf(fp, "set xlabel \"Time (ms)\"\n");
		fprintf(fp, "set ylabel \"Voltage (V)\"\n");
		fprintf(fp, "set yrange [-0.1:1.6]\n");
		fprintf(fp, "set grid xtics ytics mxtics mytics\n");
		fprintf(fp, "set style line 1 lt 2 lc rgb \"red\" lw 2\n");
		fprintf(fp, "set style line 2 lt 2 lc rgb \"blue\" lw 2\n");
		fprintf(fp, "set style line 3 lt 2 lc rgb \"green\" lw 2\n");

		if (count == 0) {
			fprintf(fp, "set terminal wxt size 1133,850\n");
		} else {
			fprintf(fp, "set terminal jpeg size 1280,960\n");
			fprintf(fp, "set output \"%s\"\n", jpeg_file);
		}

		fprintf(fp, "set multiplot;\n");
		fprintf(fp, "set size 1.0, 0.33\n");
		fprintf(fp, "set origin 0,0\n");
		fprintf(fp, "set title 'OMAP4430 VDD_CORE Voltage "
			"over time'\n");
		fprintf(fp, "plot '%s' using 1:4 ls 3 notitle with steps;\n",
			datafile);
		fprintf(fp, "set size 1.0, 0.33\n");
		fprintf(fp, "set origin 0,0.33\n");
		fprintf(fp, "set title 'OMAP4430 VDD_IVA Voltage over time'\n");
		fprintf(fp, "plot '%s' using 1:3 ls 2 notitle with steps;\n",
			datafile);
		fprintf(fp, "set size 1.0, 0.33\n");
		fprintf(fp, "set origin 0,0.66\n");
		fprintf(fp, "set title 'OMAP4430 VDD_MPU Voltage over time'\n");
		fprintf(fp, "plot '%s' using 1:2 ls 1 notitle with steps;\n",
			datafile);
		fprintf(fp, "unset multiplot\n");

		if (count == 1)
			fprintf(fp, "print \"'%s' JPEG file created.\"\n",
				jpeg_file);
		fclose(fp);
	}
	printf("GNUPlot script files generated.\n");
	printf("Type:\n");
	printf("  gnuplot -persist '%s' to draw voltage transition charts.\n",
		draw_plt_file);
	printf("  gnuplot -persist '%s' to save voltage transition charts into "
		"'%s' image file.\n\n", jpeg_plt_file, jpeg_file);

	return 0;
}
